/*	--*- c -*--
 * Copyright (C) 2009 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef H_ENSC_DHCPD_SIMPLE_DHCP_H
#define H_ENSC_DHCPD_SIMPLE_DHCP_H

#include <stdint.h>
#include <string.h>
#include <assert.h>

#include <netinet/ip.h>

/* RFC 2131, section 2 "Protocol Summary" */
struct dhcp_header {
	uint8_t		op;
	uint8_t		htype;
	uint8_t		hlen;
	uint8_t		hops;

	uint32_t	xid;
	uint16_t	secs;

	uint16_t	flags;

	in_addr_t	ciaddr;
	in_addr_t	yiaddr;
	in_addr_t	siaddr;
	in_addr_t	giaddr;

	uint8_t		chaddr[16];
	uint8_t		sname[64];
	uint8_t		file[128];
} __attribute__((__packed__));

struct dhcp_options {
	uint32_t	cookie;
	unsigned char	data[];
} __attribute__((__packed__));

struct dhcp_single_option {
	uint8_t		code;
	uint8_t		len;
	unsigned char	data[];
} __attribute__((__packed__));

enum {
	DHCP_COOKIE	=  0x63538263u,
	DHCP_BCAST	=  0x0080u,
};

enum {
	OP_BOOTREQUEST	= 1u,
	OP_BOOTREPLY	= 2u
};

enum {
	CODE_PAD	= 0u,
	CODE_END	= 255u
};

inline static size_t
dhcp_get_option_length(struct dhcp_single_option const *opt)
{
	switch (opt->code) {
	case CODE_PAD:
	case CODE_END:	return 1;
	default:	return opt->len + 2;
	}
}

inline static void
dhcp_remove_option(struct dhcp_single_option *opt,
		   struct dhcp_single_option const *end_opt)
{
	size_t			len   = dhcp_get_option_length(opt);
	void * const		start = opt;
	void const * const	end   = end_opt;

	assert(opt < end_opt);
	assert(start + len <= end);

	/* end-start  -->  character count between opt and end_opt without end_opt
	 * -len       -->  size of the option to be removed
	 * +1         -->  the end_opt */
	if (opt < end_opt && start + len <= end)
		memmove(start, start + len, end - (start + len) + 1);
}

#endif	/* H_ENSC_DHCPD_SIMPLE_DHCP_H */
