/*	--*- c -*--
 * Copyright (C) 2009 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef H_ENSC_DHCPD_SIMPLE_LIST_H
#define H_ENSC_DHCPD_SIMPLE_LIST_H

#include <stdbool.h>
#include "misc.h"

struct list_head {
	struct list_head	*next;
	struct list_head	*prev;
};

inline static void	list_init(struct list_head *head)
{
	head->prev = head;
	head->next = head;
}

inline static bool	list_empty(struct list_head const *h)
{
	return h->next == h && h->prev == h;
}

inline static void	_list_add(struct list_head *n, struct list_head *prev, struct list_head *next)
{
	n->prev = prev;
	n->next = next;
	prev->next = n;
	next->prev = n;
}

inline static void	list_add_tail(struct list_head *head, struct list_head *pos)
{
	_list_add(head, pos->prev, pos);
}

inline static void	list_add(struct list_head *head, struct list_head *pos)
{
	_list_add(head, pos, pos->next);
}

inline static void	list_remove(struct list_head *head)
{
	struct list_head	*next = head->next;
	struct list_head	*prev = head->prev;

	prev->next = next;
	next->prev = prev;

#ifndef NDEBUG
	head->next = (void *)0xdead0001;
	head->prev = (void *)0xdead0003;
#endif
}

#define list_entry(HEAD,TYPE,MEMBER)	container_of(HEAD, TYPE, MEMBER)


#endif	/* H_ENSC_DHCPD_SIMPLE_LIST_H */
