/*	--*- c -*--
 * Copyright (C) 2009 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>

#ifndef PIDFILE
#  define PIDFILE	LOCALSTATEDIR "/run/dhcpd-ptp"
#endif

enum {
	CMD_HELP = 0x1000,
	CMD_VERSION,
	CMD_DAEMON,
	CMD_SIGSTOP,
	CMD_PIDFILE
};

struct cmdline_options {
	char const		*user;
	char const		*interface;
	char const		*pidfile;
	enum { MODE_FOREGROUND, MODE_BACKGROUND, 
	       MODE_SIGSTOP }	daemon_mode;
};

static struct option const CMDLINE_OPTIONS[] = {
	{ "help",        no_argument,       NULL, CMD_HELP },
	{ "version",     no_argument,       NULL, CMD_VERSION },
	{ "pidfile",     required_argument, NULL, CMD_PIDFILE },
	{ "interface",   required_argument, NULL, 'i' },
	{ "daemon",      no_argument,       NULL, CMD_DAEMON },
	{ "sigstop",     no_argument,       NULL, CMD_SIGSTOP },
	{ "option",      required_argument, NULL, 'd' },
	{ "user",        required_argument, NULL, 'u' },
	{ NULL, 0, NULL, 0}
};

static void show_help(void)
{
	puts("Usage: dhcpd-ptp [--pidfile <filename>] [--daemon|--sigstop]\n"
	     "  [--user|-u <username>] [--interface|-i <iface>]\n"
	     "  [--option|-d <dhcp-option>]+ <ip-address>");
	exit(0);
}

static void show_version(void)
{
	puts("dhcpd-ptp " VERSION " -- simple point-to-point dhcp daemon\n\n"
	     "Copyright (C) 2009 Enrico Scholz\n"
	     "This program is free software; you may redistribute it under the terms of\n"
	     "the GNU General Public License.  This program has absolutely no warranty.");
	exit(0);
}


int main(int argc, char *argv[])
{
	struct cmdline_options	opts = {
		.user        = "dhcpd-ptp",
		.interface   = "eth0",
		.pidfile     = PIDFILE,
		.daemon_mode = MODE_FOREGROUND
	};

	while (1) {
		int		c = getopt_long(argc, argv, "+i:d:u:", CMDLINE_OPTIONS, 0);
		if (c==-1) break;

		switch (c) {
		case CMD_HELP:  	show_help();
		case CMD_VERSION:	show_version();
		case CMD_PIDFILE:	opts.pidfile   = optarg; break;
		case 'i':		opts.interface = optarg; break;
		case 'u':		opts.user      = optarg; break;
		case CMD_DAEMON:	opts.daemon_mode = MODE_BACKGROUND; break;
		case CMD_SIGSTOP:	opts.daemon_mode = MODE_SIGSTOP;    break;
		default:
			fputs("Try '--help' for more information.\n", stderr);
			return EXIT_FAILURE;
		}
	}
}
